@@ -220,6 +220,10 @@ run:
 			if (ret == UWSGI_ROUTE_GOON) {
 				goon_func = routes->func;
 			}
+
+			if (ret == -1) {
+				return UWSGI_ROUTE_BREAK;
+			}
 		}
 next:
 		subject = orig_subject;
@@ -661,6 +665,153 @@ static int uwsgi_router_signal(struct uwsgi_route *ur, char *arg) {
         return 0;
 }
 
+// chdir route
+static int uwsgi_router_chdir_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+	char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data, ur->data_len);
+	if (!ub) return UWSGI_ROUTE_BREAK;
+	if (chdir(ub->buf)) {
+		uwsgi_error("uwsgi_router_chdir_func()/chdir()");
+		uwsgi_buffer_destroy(ub);
+		return UWSGI_ROUTE_BREAK;
+	}
+	uwsgi_buffer_destroy(ub);
+        return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_chdir(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_chdir_func;
+        ur->data = arg;
+	ur->data_len = strlen(arg);
+        return 0;
+}
+
+// setapp route
+static int uwsgi_router_setapp_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data, ur->data_len);
+        if (!ub) return UWSGI_ROUTE_BREAK;
+	char *ptr = uwsgi_req_append(wsgi_req, "UWSGI_APPID", 11, ub->buf, ub->pos);
+	if (!ptr) {
+                uwsgi_buffer_destroy(ub);
+                return UWSGI_ROUTE_BREAK;
+        }
+	wsgi_req->appid = ptr;
+	wsgi_req->appid_len = ub->pos;
+        uwsgi_buffer_destroy(ub);
+        return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_setapp(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_setapp_func;
+        ur->data = arg;
+        ur->data_len = strlen(arg);
+        return 0;
+}
+
+// sethome route
+static int uwsgi_router_sethome_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data, ur->data_len);
+        if (!ub) return UWSGI_ROUTE_BREAK;
+        char *ptr = uwsgi_req_append(wsgi_req, "UWSGI_HOME", 10, ub->buf, ub->pos);
+        if (!ptr) {
+                uwsgi_buffer_destroy(ub);
+                return UWSGI_ROUTE_BREAK;
+        }
+        wsgi_req->home = ptr;
+        wsgi_req->home_len = ub->pos;
+        uwsgi_buffer_destroy(ub);
+        return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_sethome(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_sethome_func;
+        ur->data = arg;
+        ur->data_len = strlen(arg);
+        return 0;
+}
+
+// setfile route
+static int uwsgi_router_setfile_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data, ur->data_len);
+        if (!ub) return UWSGI_ROUTE_BREAK;
+        char *ptr = uwsgi_req_append(wsgi_req, "UWSGI_HOME", 10, ub->buf, ub->pos);
+        if (!ptr) {
+                uwsgi_buffer_destroy(ub);
+                return UWSGI_ROUTE_BREAK;
+        }
+        wsgi_req->file = ptr;
+        wsgi_req->file_len = ub->pos;
+	wsgi_req->dynamic = 1;
+        uwsgi_buffer_destroy(ub);
+        return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_setfile(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_setfile_func;
+        ur->data = arg;
+        ur->data_len = strlen(arg);
+        return 0;
+}
+
+
+// setprocname route
+static int uwsgi_router_setprocname_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data, ur->data_len);
+        if (!ub) return UWSGI_ROUTE_BREAK;
+	uwsgi_set_processname(ub->buf);
+        uwsgi_buffer_destroy(ub);
+        return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_setprocname(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_setprocname_func;
+        ur->data = arg;
+        ur->data_len = strlen(arg);
+        return 0;
+}
+
+// alarm route
+static int uwsgi_router_alarm_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+
+        struct uwsgi_buffer *ub_alarm = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data, ur->data_len);
+        if (!ub_alarm) return UWSGI_ROUTE_BREAK;
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data2, ur->data2_len);
+        if (!ub) {
+		uwsgi_buffer_destroy(ub_alarm);
+		return UWSGI_ROUTE_BREAK;
+	}
+	uwsgi_alarm_trigger(ub_alarm->buf, ub->buf, ub->pos);	
+        uwsgi_buffer_destroy(ub_alarm);
+        uwsgi_buffer_destroy(ub);
+        return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_alarm(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_alarm_func;
+	char *space = strchr(arg, ' ');
+	if (!space) {
+		return -1;
+	}
+	*space = 0;
+        ur->data = arg;
+        ur->data_len = strlen(arg);
+        ur->data2 = space+1;
+        ur->data2_len = strlen(ur->data2);
+        return 0;
+}
+
+
 
 // send route
 static int uwsgi_router_send_func(struct wsgi_request *wsgi_req, struct uwsgi_route *route) {
@@ -780,6 +931,119 @@ static int uwsgi_route_condition_equal(struct wsgi_request *wsgi_req, struct uws
         return 0;
 }
 
+static int uwsgi_route_condition_higher(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char *semicolon = memchr(ur->subject_str, ';', ur->subject_str_len);
+        if (!semicolon) return 0;
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, ur->subject_str, semicolon - ur->subject_str);
+        if (!ub) return -1;
+
+        struct uwsgi_buffer *ub2 = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, semicolon+1, ur->subject_str_len - ((semicolon+1) - ur->subject_str));
+        if (!ub2) {
+                uwsgi_buffer_destroy(ub);
+                return -1;
+        }
+
+	long num1 = strtol(ub->buf, NULL, 10);
+	long num2 = strtol(ub2->buf, NULL, 10);
+        if(num1 > num2) {
+                uwsgi_buffer_destroy(ub);
+                uwsgi_buffer_destroy(ub2);
+                return 1;
+        }
+        uwsgi_buffer_destroy(ub);
+        uwsgi_buffer_destroy(ub2);
+        return 0;
+}
+
+static int uwsgi_route_condition_higherequal(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char *semicolon = memchr(ur->subject_str, ';', ur->subject_str_len);
+        if (!semicolon) return 0;
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, ur->subject_str, semicolon - ur->subject_str);
+        if (!ub) return -1;
+
+        struct uwsgi_buffer *ub2 = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, semicolon+1, ur->subject_str_len - ((semicolon+1) - ur->subject_str));
+        if (!ub2) {
+                uwsgi_buffer_destroy(ub);
+                return -1;
+        }
+
+        long num1 = strtol(ub->buf, NULL, 10);
+        long num2 = strtol(ub2->buf, NULL, 10);
+        if(num1 >= num2) {
+                uwsgi_buffer_destroy(ub);
+                uwsgi_buffer_destroy(ub2);
+                return 1;
+        }
+        uwsgi_buffer_destroy(ub);
+        uwsgi_buffer_destroy(ub2);
+        return 0;
+}
+
+
+static int uwsgi_route_condition_lower(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char *semicolon = memchr(ur->subject_str, ';', ur->subject_str_len);
+        if (!semicolon) return 0;
+        
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, ur->subject_str, semicolon - ur->subject_str);
+        if (!ub) return -1;
+
+        struct uwsgi_buffer *ub2 = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, semicolon+1, ur->subject_str_len - ((semicolon+1) - ur->subject_str));
+        if (!ub2) {
+                uwsgi_buffer_destroy(ub);
+                return -1;
+        }
+
+        long num1 = strtol(ub->buf, NULL, 10);
+        long num2 = strtol(ub2->buf, NULL, 10);
+        if(num1 < num2) {
+                uwsgi_buffer_destroy(ub);
+                uwsgi_buffer_destroy(ub2);
+                return 1;
+        }
+        uwsgi_buffer_destroy(ub);
+        uwsgi_buffer_destroy(ub2);
+        return 0;
+}
+
+static int uwsgi_route_condition_lowerequal(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char *semicolon = memchr(ur->subject_str, ';', ur->subject_str_len);
+        if (!semicolon) return 0;
+        
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, ur->subject_str, semicolon - ur->subject_str);
+        if (!ub) return -1;
+
+        struct uwsgi_buffer *ub2 = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, semicolon+1, ur->subject_str_len - ((semicolon+1) - ur->subject_str));
+        if (!ub2) {
+                uwsgi_buffer_destroy(ub);
+                return -1;
+        }
+
+        long num1 = strtol(ub->buf, NULL, 10);
+        long num2 = strtol(ub2->buf, NULL, 10);
+        if(num1 <= num2) {
+                uwsgi_buffer_destroy(ub);
+                uwsgi_buffer_destroy(ub2);
+                return 1;
+        }
+        uwsgi_buffer_destroy(ub);
+        uwsgi_buffer_destroy(ub2);
+        return 0;
+}
+
+#ifdef UWSGI_SSL
+static int uwsgi_route_condition_lord(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+	struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, ur->subject_str, ur->subject_str_len);
+        if (!ub) return -1;
+        int ret = uwsgi_legion_i_am_the_lord(ub->buf);
+        uwsgi_buffer_destroy(ub);
+        return ret;
+}
+#endif
+
+
+
 static int uwsgi_route_condition_startswith(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
         char *semicolon = memchr(ur->subject_str, ';', ur->subject_str_len);
         if (!semicolon) return 0;
@@ -865,6 +1129,70 @@ static int uwsgi_route_condition_isexec(struct wsgi_request *wsgi_req, struct uw
         return 0;
 }
 
+static char *uwsgi_route_var_uwsgi(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
+	char *ret = NULL;
+	if (!uwsgi_strncmp(key, keylen, "wid", 3)) {
+		ret = uwsgi_num2str(uwsgi.mywid);
+		*vallen = strlen(ret);
+	}
+	else if (!uwsgi_strncmp(key, keylen, "pid", 3)) {
+		ret = uwsgi_num2str(uwsgi.mypid);
+		*vallen = strlen(ret);
+	}
+
+	return ret;
+}
+
+static char *uwsgi_route_var_time(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
+        char *ret = NULL;
+        if (!uwsgi_strncmp(key, keylen, "unix", 4)) {
+                ret = uwsgi_num2str(uwsgi_now());
+                *vallen = strlen(ret);
+        }
+        return ret;
+}
+
+#ifdef UWSGI_MATHEVAL
+static char *uwsgi_route_var_math(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
+	char *ret = NULL;
+	// avoid crash	
+	if (!wsgi_req->var_cnt) return NULL;
+	// we make a bit of fun here, we do a copy of the vars buffer (+1 byte for final zero) and zeor-pad all of the strings
+	char *vars_buf = uwsgi_malloc(wsgi_req->uh->pktsize + keylen + 1);
+	char **names = uwsgi_malloc(sizeof(char *) * (wsgi_req->var_cnt/2));
+	double *values = uwsgi_calloc(sizeof(double) * (wsgi_req->var_cnt/2));
+	int i,j = 0;
+	char *ptr = vars_buf;
+	for (i = wsgi_req->var_cnt-1; i > 0; i -= 2) {
+		memcpy(ptr, wsgi_req->hvec[i-1].iov_base, wsgi_req->hvec[i-1].iov_len);	
+		names[j] = ptr;
+		ptr += wsgi_req->hvec[i-1].iov_len;
+		*ptr++=0;
+		char *num = ptr;
+		memcpy(ptr, wsgi_req->hvec[i].iov_base, wsgi_req->hvec[i].iov_len);
+		ptr += wsgi_req->hvec[i].iov_len;
+                *ptr++=0;
+		values[j] = strtod(num, NULL);
+		j++;
+        }
+
+	char *expr = ptr;
+	memcpy(ptr, key, keylen); ptr += keylen;
+	*ptr++=0;
+
+	void *e = evaluator_create(expr);
+        if (!e) goto end;
+        double n = evaluator_evaluate(e, j, names, values);
+        evaluator_destroy(e);
+	ret = uwsgi_num2str((int)n);
+	*vallen = strlen(ret);
+end:
+	free(vars_buf);
+	free(names);
+	free(values);
+	return ret;
+}
+#endif
 
 // register embedded routers
 void uwsgi_register_embedded_routers() {
@@ -882,6 +1210,12 @@ void uwsgi_register_embedded_routers() {
         uwsgi_register_router("signal", uwsgi_router_signal);
         uwsgi_register_router("send", uwsgi_router_send);
         uwsgi_register_router("send-crnl", uwsgi_router_send_crnl);
+        uwsgi_register_router("chdir", uwsgi_router_chdir);
+        uwsgi_register_router("setapp", uwsgi_router_setapp);
+        uwsgi_register_router("sethome", uwsgi_router_sethome);
+        uwsgi_register_router("setfile", uwsgi_router_setfile);
+        uwsgi_register_router("setprocname", uwsgi_router_setprocname);
+        uwsgi_register_router("alarm", uwsgi_router_alarm);
 
         uwsgi_register_route_condition("exists", uwsgi_route_condition_exists);
         uwsgi_register_route_condition("isfile", uwsgi_route_condition_isfile);
@@ -896,11 +1230,30 @@ void uwsgi_register_embedded_routers() {
         uwsgi_register_route_condition("endswith", uwsgi_route_condition_endswith);
         uwsgi_register_route_condition("regexp", uwsgi_route_condition_regexp);
         uwsgi_register_route_condition("re", uwsgi_route_condition_regexp);
+        uwsgi_register_route_condition("ishigher", uwsgi_route_condition_higher);
+        uwsgi_register_route_condition(">", uwsgi_route_condition_higher);
+        uwsgi_register_route_condition("islower", uwsgi_route_condition_lower);
+        uwsgi_register_route_condition("<", uwsgi_route_condition_lower);
+        uwsgi_register_route_condition("ishigherequal", uwsgi_route_condition_higherequal);
+        uwsgi_register_route_condition(">=", uwsgi_route_condition_higherequal);
+        uwsgi_register_route_condition("islowerequal", uwsgi_route_condition_lowerequal);
+        uwsgi_register_route_condition("<=", uwsgi_route_condition_lowerequal);
+#ifdef UWSGI_SSL
+        uwsgi_register_route_condition("lord", uwsgi_route_condition_lord);
+#endif
 
         uwsgi_register_route_condition("empty", uwsgi_route_condition_empty);
 
         uwsgi_register_route_var("cookie", uwsgi_get_cookie);
         uwsgi_register_route_var("qs", uwsgi_get_qs);
+        struct uwsgi_route_var *urv = uwsgi_register_route_var("uwsgi", uwsgi_route_var_uwsgi);
+	urv->need_free = 1;
+        urv = uwsgi_register_route_var("time", uwsgi_route_var_time);
+	urv->need_free = 1;
+#ifdef UWSGI_MATHEVAL
+        urv = uwsgi_register_route_var("math", uwsgi_route_var_math);
+	urv->need_free = 1;
+#endif
 }
 
 struct uwsgi_router *uwsgi_register_router(char *name, int (*func) (struct uwsgi_route *, char *)) {
