@@ -177,87 +177,6 @@ int uwsgi_add_file_monitor(uint8_t sig, char *filename) {
 
 }
 
-struct uwsgi_probe *uwsgi_probe_register(struct uwsgi_probe **up, char *name, int (*func) (int, struct uwsgi_signal_probe *)) {
-
-	struct uwsgi_probe *uwsgi_up = *up, *old_up;
-
-	if (!uwsgi_up) {
-		*up = uwsgi_malloc(sizeof(struct uwsgi_probe));
-		uwsgi_up = *up;
-	}
-	else {
-		while (uwsgi_up) {
-			old_up = uwsgi_up;
-			uwsgi_up = uwsgi_up->next;
-		}
-
-		uwsgi_up = uwsgi_malloc(sizeof(struct uwsgi_probe));
-		old_up->next = uwsgi_up;
-	}
-
-	uwsgi_up->name = name;
-	uwsgi_up->func = func;
-	uwsgi_up->next = NULL;
-
-	uwsgi_log("registered new probe \"%s\" at %p\n", name, uwsgi_up);
-
-	return uwsgi_up;
-}
-
-
-int uwsgi_add_probe(uint8_t sig, char *kind, char *args, int timeout, int freq) {
-
-	uwsgi_lock(uwsgi.probe_table_lock);
-
-	if (ushared->probes_cnt < MAX_PROBES) {
-
-		struct uwsgi_probe *up = uwsgi.probes;
-		while (up) {
-			if (!strcmp(up->name, kind)) {
-				break;
-			}
-			up = up->next;
-		}
-
-		if (!up) {
-			uwsgi_log("unable to find probe \"%s\" !!!\n", kind);
-			uwsgi_unlock(uwsgi.probe_table_lock);
-			return -1;
-		}
-
-		// fill the probe table
-		ushared->probes[ushared->probes_cnt].func = up->func;
-		strncpy(ushared->probes[ushared->probes_cnt].args, args, 1024 - 1);
-		ushared->probes[ushared->probes_cnt].registered = 0;
-		ushared->probes[ushared->probes_cnt].sig = sig;
-		ushared->probes[ushared->probes_cnt].fd = -1;
-		ushared->probes[ushared->probes_cnt].state = 0;
-		ushared->probes[ushared->probes_cnt].last_event = 0;
-		ushared->probes[ushared->probes_cnt].data = NULL;
-		ushared->probes[ushared->probes_cnt].cycles = 0;
-		ushared->probes[ushared->probes_cnt].bad = 0;
-
-		if (!timeout) {
-			timeout = uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT];
-		}
-		ushared->probes[ushared->probes_cnt].timeout = timeout;
-		if (!freq) {
-			freq = 1;
-		}
-		ushared->probes[ushared->probes_cnt].freq = freq;
-		ushared->probes_cnt++;
-	}
-	else {
-		uwsgi_log("you can register max %d probes !!!\n", MAX_PROBES);
-		uwsgi_unlock(uwsgi.probe_table_lock);
-		return -1;
-	}
-
-	uwsgi_unlock(uwsgi.probe_table_lock);
-
-	return 0;
-}
-
 int uwsgi_add_timer(uint8_t sig, int secs) {
 
 	if (!uwsgi.master_process) return -1;
