@@ -1525,6 +1525,25 @@ int uwsgi_logic_opt_for_glob(char *key, char *value) {
         return 1;
 }
 
+int uwsgi_logic_opt_for_times(char *key, char *value) {
+
+	int num = atoi(uwsgi.logic_opt_data);
+        int i;
+	char str_num[11];
+
+        for (i = 1; i <= num; i++) {
+		int ret = uwsgi_num2str2(i, str_num);
+		// security check
+		if (ret < 0 || ret > 11) {
+			exit(1);
+		}
+                add_exported_option(key, uwsgi_substitute(value, "%(_)", str_num), 0);
+        }
+
+        return 1;
+}
+
+
 
 void add_exported_option(char *key, char *value, int configured) {
 
@@ -3054,14 +3073,17 @@ void http_url_decode(char *buf, uint16_t * len, char *dst) {
 
 }
 
+/*
+	we scan the table in reverse, as updated values are at the end
+*/
 char *uwsgi_get_var(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t * len) {
 
 	int i;
 
-	for (i = 0; i < wsgi_req->var_cnt; i += 2) {
-		if (!uwsgi_strncmp(key, keylen, wsgi_req->hvec[i].iov_base, wsgi_req->hvec[i].iov_len)) {
-			*len = wsgi_req->hvec[i + 1].iov_len;
-			return wsgi_req->hvec[i + 1].iov_base;
+	for (i = wsgi_req->var_cnt-1; i > 0; i -= 2) {
+		if (!uwsgi_strncmp(key, keylen, wsgi_req->hvec[i-1].iov_base, wsgi_req->hvec[i-1].iov_len)) {
+			*len = wsgi_req->hvec[i].iov_len;
+			return wsgi_req->hvec[i].iov_base;
 		}
 	}
 
