@@ -1,6 +1,6 @@
 # uWSGI build system
 
-uwsgi_version = '1.9.5'
+uwsgi_version = '1.9.6'
 
 import os
 import re
@@ -66,11 +66,9 @@ report['routing'] = False
 report['capabilities'] = False
 report['yaml'] = False
 report['json'] = False
-report['ldap'] = False
 report['ssl'] = False
 report['zeromq'] = False
 report['xml'] = False
-report['sqlite3'] = False
 report['debug'] = False
 report['plugin_dir'] = False
 report['zlib'] = False
@@ -524,6 +522,8 @@ class uConf(object):
         # add -fno-strict-aliasing only on python2 and gcc < 4.3
         if gcc_major >= 4:
             self.cflags = self.cflags + [ '-Wextra', '-Wno-unused-parameter', '-Wno-missing-field-initializers' ]
+        if (gcc_major == 4 and gcc_minor >= 8) or gcc_major > 4:
+            self.cflags.append('-Wno-format')
 
         self.ldflags = os.environ.get("LDFLAGS", "").split()
         self.libs = ['-lpthread', '-lm', '-rdynamic']
@@ -971,19 +971,6 @@ class uConf(object):
         if has_json:
             report['json'] = True
 
-        if self.get('ldap'):
-            if self.get('ldap') == 'auto':
-                if self.has_include('ldap.h'):
-                    self.cflags.append("-DUWSGI_LDAP")
-                    self.gcc_list.append('core/ldap')
-                    self.libs.append('-lldap')
-                    report['ldap'] = True
-            else:
-                self.cflags.append("-DUWSGI_LDAP")
-                self.gcc_list.append('core/ldap')
-                self.libs.append('-lldap')
-                report['ldap'] = True
-
         if self.get('ssl'):
             if self.get('ssl') == 'auto':
                 if self.has_include('openssl/ssl.h'):
@@ -1052,20 +1039,6 @@ class uConf(object):
                 self.gcc_list.append('core/xmlconf')
                 report['xml'] = 'expat'
 
-        if self.get('sqlite3'):
-            if self.get('sqlite3') == 'auto':
-                if self.has_include('sqlite3.h'):
-                    self.cflags.append("-DUWSGI_SQLITE3")
-                    self.libs.append('-lsqlite3')
-                    self.gcc_list.append('core/sqlite3')
-                    report['sqlite3'] = True
-            else:
-                self.cflags.append("-DUWSGI_SQLITE3")
-                self.libs.append('-lsqlite3')
-                self.gcc_list.append('core/sqlite3')
-                report['sqlite3'] = True
-
-
         if self.get('plugin_dir'):
             self.cflags.append('-DUWSGI_PLUGIN_DIR=\\"%s\\"' % self.get('plugin_dir'))
             report['plugin_dir'] = self.get('plugin_dir')
